@@ -1,4 +1,4 @@
-%% Copyright (C) 2014 Colin B. Macdonald
+%% Copyright (C) 2014, 2016 Colin B. Macdonald
 %%
 %% This file is part of OctSymPy.
 %%
@@ -29,13 +29,10 @@
 
 function B = flipud(A)
 
-  cmd = { '(A,) = _ins' ...
-          'if not A.is_Matrix:' ...
-          '    A = sp.Matrix([A])' ...
-          'm, _ = A.shape' ...
-          'for i in range(0, int(m/2)):' ...
-          '    A.row_swap(i, m-i-1)' ...
-          'return A,' };
+  cmd = { 'A, = _ins'
+          'if A is None or not A.is_Matrix:'
+          '    A = sp.Matrix([A])'
+          'return A[::-1, :]' };
 
   B = python_cmd (cmd, sym(A));
 
@@ -60,4 +57,3 @@ end
 %! % scalar
 %! syms x
 %! assert (isequal (flipud(x), x))
-
