@@ -1,4 +1,4 @@
-%% Copyright (C) 2014, 2015 Colin B. Macdonald
+%% Copyright (C) 2014-2016 Colin B. Macdonald
 %%
 %% This file is part of OctSymPy.
 %%
@@ -18,25 +18,26 @@
 
 %% -*- texinfo -*-
 %% @documentencoding UTF-8
-%% @deftypefn  {Function File} {@var{g} =} function_handle (@var{f})
-%% @deftypefnx {Function File} {@var{g} =} function_handle (@var{f1}, @dots{}, @var{fn})
-%% @deftypefnx {Function File} {@var{g} =} function_handle (@dots{}, @var{param}, @var{value})
-%% @deftypefnx {Function File} {@var{g} =} function_handle (@dots{}, 'vars', [@var{x} @dots{} @var{z}])
-%% @deftypefnx {Function File} {@dots{}} function_handle (@dots{}, 'file', @var{filename})
-%% @deftypefnx {Function File} {@dots{}} function_handle (@dots{}, 'outputs', [@var{o1} @dots{} @var{on}])
+%% @defmethod  @@sym function_handle (@var{f})
+%% @defmethodx @@sym function_handle (@var{f1}, @dots{}, @var{fn})
+%% @defmethodx @@sym function_handle (@dots{}, @var{param}, @var{value})
+%% @defmethodx @@sym function_handle (@dots{}, 'vars', [@var{x} @dots{} @var{z}])
+%% @defmethodx @@sym function_handle (@dots{}, 'file', @var{filename})
+%% @defmethodx @@sym function_handle (@dots{}, 'outputs', [@var{o1} @dots{} @var{on}])
 %% Convert symbolic expression into a standard function.
 %%
 %% This can make anonymous functions from symbolic expressions:
 %% @example
 %% @group
-%% >> syms x y
-%% >> f = x^2 + sin(y)
-%%    @result{} f = (sym)
-%%         2
-%%        x  + sin(y)
-%% >> h = function_handle(f)
-%%    @result{} h =
-%%      @@(x, y) x .^ 2 + sin (y)
+%% syms x y
+%% f = x^2 + sin(y)
+%%   @result{} f = (sym)
+%%        2
+%%       x  + sin(y)
+%% h = function_handle(f)
+%%   @result{} h = @@(x, y) x .^ 2 + sin (y)
+%% h(2, pi/2)
+%%   @result{} ans =  5
 %% @end group
 %% @end example
 %%
@@ -45,26 +46,25 @@
 %% specifies the third output (rather than the input):
 %% @example
 %% @group
-%% >> h = function_handle(x^2, 5*x, x);
-%% >> [a, b, c] = h(2)
-%%    @result{} a = 4
-%%    @result{} b = 10
-%%    @result{} c = 2
+%% h = function_handle(x^2, 5*x, x);
+%% [a, b, c] = h(2)
+%%   @result{} a = 4
+%%   @result{} b = 10
+%%   @result{} c = 2
 %% @end group
 %% @end example
 %%
 %% The order and number of inputs can be specified:
 %% @example
 %% @group
-%% >> syms x y z
-%% >> h = function_handle(f, 'vars', [z y x])
-%%    @result{} h =
-%%      @@(z, y, x) x .^ 2 + sin (y)
+%% syms x y z
+%% h = function_handle(f, 'vars', [z y x])
+%%   @result{} h = @@(z, y, x) x .^ 2 + sin (y)
 %% @end group
 %% @end example
 %%
 %% For compatibility with the Symbolic Math Toolbox in Matlab, we
-%% provide a synonym: @xref{matlabFunction}
+%% provide a synonym: @pxref{@@sym/matlabFunction}
 %%
 %% OctSymPy can also generate an @code{.m} file from a symbolic
 %% expression by passing the keyword @code{file} with a string
@@ -73,30 +73,24 @@
 %% Passing an empty @var{filename} creates an anonymous function:
 %% @example
 %% @group
-%% >> h = function_handle(f, 'file', '')
-%%    @result{} h =
-%%      @@(x, y) x .^ 2 + sin (y)
+%% h = function_handle(f, 'file', '')
+%%   @result{} h = @@(x, y) x .^ 2 + sin (y)
 %% @end group
 %% @end example
 %%
-%% FIXME: naming outputs with @var{PARAM} as
-%% @code{outputs} not implemented.
+%% FIXME: naming outputs with @var{PARAM} as @code{outputs}
+%% not implemented.
 %%
 %% FIXME: does not ``optimize'' code, for example, using common
 %% subexpression elimination.
 %%
-%% The routine relies on code generation features added to SymPy
-%% 0.7.6.  On earlier versions, the workaround only works for
-%% very simple expressions such as polynomials and trig functions.
-%%
-%% @seealso{ccode, fortran, latex, matlabFunction}
-%% @end deftypefn
+%% @seealso{@@sym/ccode, @@sym/fortran, @@sym/latex, @@sym/matlabFunction}
+%% @end defmethod
 
-%% Author: Colin B. Macdonald
-%% Keywords: symbolic
 
 function f = function_handle(varargin)
 
+  % We use the private/codegen function only for its input parsing
   [flg, meh] = codegen(varargin{:}, 'lang', 'octave');
   assert(flg == -1);
   [Nin, inputs, inputstr, Nout, param] = deal(meh{:});
@@ -114,31 +108,42 @@ function f = function_handle(varargin)
             '    return (False, str(e))' ...
             'return (True, out)' };
 
-    % if filename ends with .m, do not add another
-    if strcmpi(param.fname(end-1:end), '.m')
-      param.fname = param.fname(1:end-2);
-    end
-
-    fname2 = param.fname; fcnname = param.fname;
-    % was old note about findsymbols vs symvar: not relevant
-    [worked, out] = python_cmd (cmd, varargin(1:Nout), fcnname, fname2, param.show_header, inputs);
+    [fcnpath, fcnname, fcnext] = fileparts(param.fname);
+    [worked, out] = python_cmd (cmd, varargin(1:Nout), fcnname, fcnname, param.show_header, inputs);
 
     if (~worked)
       if (strcmp(out, 'Language ''octave'' is not supported.'))
-	error('function_handle: your SymPy has no octave codegen, cannot workaround');
+        error('function_handle: your SymPy has no octave codegen');
       else
-	ou
-	error('function_handle: Some other error from SymPy code gen?  file a bug!');
+        out
+        error('function_handle: Some other error from SymPy code gen?  file a bug!');
       end
     end
     M.name = out{1}{1};
     M.code = out{1}{2};
 
-    [fid,msg] = fopen(M.name, 'w');
+    assert (strcmp (M.name, [fcnname '.m']), 'sanity check failed: names should match');
+
+    file_to_write = fullfile(fcnpath, [fcnname '.m']);
+    [fid,msg] = fopen(file_to_write, 'w');
     assert(fid > -1, msg)
     fprintf(fid, '%s', M.code)
     fclose(fid);
-    fprintf('Wrote file %s.\n', M.name);
+    fprintf('Wrote file %s.\n', file_to_write);
+
+    % FIXME: Check upstream to rehash the files correctly once created
+    % Due to an upstream bug in octave on windows, we have to wait for the file to be loaded.
+    % See https://savannah.gnu.org/bugs/?31080 for more information...\n
+    if (exist('OCTAVE_VERSION', 'builtin') && ispc())
+      fprintf('Workaround savannah.gnu.org/bugs/?31080: waiting for %s... ', fcnname);
+      fflush(stdout);
+      while (exist(fcnname) == 0)
+        rehash()
+        pause(1)
+      end
+      fprintf('Found!\n');
+    end
+
     f = str2func(fcnname);
 
   else % output function handle
@@ -157,23 +162,8 @@ function f = function_handle(varargin)
               '    return (False, "expected symbols-to-declare to be empty")' ...
               'return (True, s)' };
       [worked, codestr] = python_cmd (cmd, expr);
-      %worked = false;
-      if (worked)
-        codestr = vectorize(codestr);
-      else
-        %% SymPy 0.7.5 has no octave_code command
-        % Use a crude workaround (e.g., Abs, ceiling will fail).
-        if (str2num(strrep(python_cmd ('return sp.__version__,'),'.',''))<=75 ...
-            && strcmp(codestr, 'global name ''octave_code'' is not defined'))
-          warning('OctSymPy:function_handle:nocodegen', ...
-                  'function_handle: your SymPy has no octave codegen: partial workaround');
-          codestr = expr.flat;
-          % Matlab: ** to ^ substition.  On Octave, vectorize does this
-          codestr = strrep(codestr, '**', '^');
-          codestr = vectorize(codestr);
-        else
-          error('function_handle: python codegen failed: %s', codestr)
-        end
+      if (~worked)
+        error('function_handle: python codegen failed: %s', codestr)
       end
       exprstr{i} = codestr;
     end
@@ -199,66 +189,53 @@ end
 
 %!test
 %! % basic test
-%! s = warning('off', 'OctSymPy:function_handle:nocodegen');
 %! h = function_handle(2*x);
-%! warning(s)
 %! assert(isa(h, 'function_handle'))
 %! assert(h(3)==6)
 
 %!test
 %! % autodetect inputs
-%! s = warning('off', 'OctSymPy:function_handle:nocodegen');
 %! h = function_handle(2*x*y, x+y);
-%! warning(s)
 %! [t1, t2] = h(3,5);
 %! assert(t1 == 30 && t2 == 8)
 
 %!test
 %! % specified inputs
-%! s = warning('off', 'OctSymPy:function_handle:nocodegen');
 %! h = function_handle(2*x*y, 'vars', [x y]);
 %! assert(h(3,5)==30)
 %! h = function_handle(2*x*y, x+y, 'vars', [x y]);
-%! warning(s)
 %! [t1, t2] = h(3,5);
 %! assert(t1 == 30 && t2 == 8)
 
 %!test
 %! % cell arrays for vars list
-%! s = warning('off', 'OctSymPy:function_handle:nocodegen');
 %! h = function_handle(2*x*y, x+y, 'vars', {x y});
 %! [t1, t2] = h(3,5);
 %! assert(t1 == 30 && t2 == 8)
 %! h = function_handle(2*x*y, x+y, 'vars', {'x' 'y'});
-%! warning(s)
 %! [t1, t2] = h(3,5);
 %! assert(t1 == 30 && t2 == 8)
 
 %!test
 %! % cell arrays specfies order, overriding symvar order
-%! s = warning('off', 'OctSymPy:function_handle:nocodegen');
 %! h = function_handle(x*y, 12/y, 'vars', {y x});
 %! [t1, t2] = h(3, 6);
 %! assert(t1 == 18 && t2 == 4)
 %! h = function_handle(x*y, 12/y, 'vars', [y x]);
-%! warning(s)
 %! [t1, t2] = h(3, 6);
 %! assert(t1 == 18 && t2 == 4)
 
 %!test
 %! % cell arrays specfies order, overriding symvar order
-%! s = warning('off', 'OctSymPy:function_handle:nocodegen');
 %! h = function_handle(x*y, 12/y, 'vars', {y x});
 %! [t1, t2] = h(3, 6);
 %! assert(t1 == 18 && t2 == 4)
 %! h = function_handle(x*y, 12/y, 'vars', [y x]);
-%! warning(s)
 %! [t1, t2] = h(3, 6);
 %! assert(t1 == 18 && t2 == 4)
 
-%!xtest
+%!test
 %! % Functions with different names in Sympy.
-%! % (will fail unless Sympy has Octave codegen)
 %! f = abs(x);  % becomes Abs(x)
 %! h = function_handle(f);
 %! assert(h(-10) == 10)
@@ -268,40 +245,64 @@ end
 
 %!test
 %! % 'file' with empty filename returns handle
-%! s = warning('off', 'OctSymPy:function_handle:nocodegen');
 %! h = function_handle(2*x*y, 'file', '');
 %! assert(isa(h, 'function_handle'))
 %! assert(h(3,5)==30)
 %! h = function_handle(2*x*y, 'vars', {x y}, 'file', '');
-%! warning(s)
 %! assert(isa(h, 'function_handle'))
 %! assert(h(3,5)==30)
 
-%!xtest
+%!test
 %! % output to disk
-%! % (will fail unless Sympy has Octave codegen)
-%! f = function_handle(2*x*y, 2^x, 'vars', {x y z}, 'file', 'temp_test_output1');
+%! fprintf('\n')
+%! if (exist ('OCTAVE_VERSION', 'builtin'))
+%!   temp_file = tempname('', 'oct_');
+%! else
+%!   temp_file = tempname();
+%! end
+%! % allow loading function from temp_file
+%! [temp_path, ans, ans] = fileparts(temp_file);
+%! addpath(temp_path);
+%! f = function_handle(2*x*y, 2^x, 'vars', {x y z}, 'file', temp_file);
 %! assert( isa(f, 'function_handle'))
 %! [a,b] = f(10,20,30);
 %! assert (isnumeric (a) && isnumeric (b))
 %! assert (a == 400)
 %! assert (b == 1024)
-%! delete('temp_test_output1.m')
+%! if (exist ('OCTAVE_VERSION', 'builtin'))
+%!   assert (unlink([temp_file '.m']) == 0)
+%! else
+%!   delete ([temp_file '.m'])
+%! end
+%! % remove temp_path from load path
+%! rmpath(temp_path);
 
-%!xtest
+%!test
 %! % output to disk: also works with .m specified
-%! % (will fail unless Sympy has Octave codegen)
-%! f = function_handle(2*x*y, 2^x, 'vars', {x y z}, 'file', 'temp_test_output2.m');
+%! if (exist ('OCTAVE_VERSION', 'builtin'))
+%!   temp_file = [tempname('', 'oct_') '.m'];
+%! else
+%!   temp_file = [tempname() '.m'];
+%! end
+%! % allow loading function from temp_file
+%! [temp_path, ans, ans] = fileparts(temp_file);
+%! addpath(temp_path);
+%! f = function_handle(2*x*y, 2^x, 'vars', {x y z}, 'file', temp_file);
 %! assert( isa(f, 'function_handle'))
 %! [a,b] = f(10,20,30);
 %! assert (isnumeric (a) && isnumeric (b))
 %! assert (a == 400)
 %! assert (b == 1024)
-%! delete('temp_test_output2.m')
+%! if (exist ('OCTAVE_VERSION', 'builtin'))
+%!   assert (unlink(temp_file) == 0)
+%! else
+%!   delete (temp_file)
+%! end
+%! % remove temp_path from load path
+%! rmpath(temp_path);
 
-%!xtest
+%!test
 %! % non-scalar outputs
-%! % (will fail unless Sympy has Octave codegen)
 %! H = [x y z];
 %! M = [x y; z 16];
 %! V = [x;y;z];
@@ -311,19 +312,32 @@ end
 %! assert(isequal(t2, [1 2; 3 16]))
 %! assert(isequal(t3, [1;2;3]))
 
-%!xtest
+%!test
 %! % non-scalar outputs in .m files
-%! % (will fail unless Sympy has Octave codegen)
 %! H = [x y z];
 %! M = [x y; z 16];
 %! V = [x;y;z];
-%! h = function_handle(H, M, V, 'vars', {x y z}, 'file', 'temp_test_output3');
+%! if (exist ('OCTAVE_VERSION', 'builtin'))
+%!   temp_file = tempname('', 'oct_');
+%! else
+%!   temp_file = tempname();
+%! end
+%! % allow loading function from temp_file
+%! [temp_path, ans, ans] = fileparts(temp_file);
+%! addpath(temp_path);
+%! h = function_handle(H, M, V, 'vars', {x y z}, 'file', temp_file);
 %! assert( isa(h, 'function_handle'))
 %! [t1,t2,t3] = h(1,2,3);
 %! assert(isequal(t1, [1 2 3]))
 %! assert(isequal(t2, [1 2; 3 16]))
 %! assert(isequal(t3, [1;2;3]))
-%! delete('temp_test_output3.m')
+%! if (exist ('OCTAVE_VERSION', 'builtin'))
+%!   assert (unlink([temp_file '.m']) == 0)
+%! else
+%!   delete ([temp_file '.m'])
+%! end
+%! % remove temp_path from load path
+%! rmpath(temp_path);
 
 %!test
 %! % order of outputs is lexiographic
