@@ -1,4 +1,4 @@
-%% Copyright (C) 2014, 2015 Colin B. Macdonald
+%% Copyright (C) 2014-2016 Colin B. Macdonald
 %%
 %% This file is part of OctSymPy.
 %%
@@ -18,66 +18,87 @@
 
 %% -*- texinfo -*-
 %% @documentencoding UTF-8
-%% @deftypefn  {Function File} {@var{r} =} sympref ()
-%% @deftypefnx {Function File} {@var{r} =} sympref (@var{cmd})
-%% @deftypefnx {Function File} {} sympref @var{cmd}
-%% @deftypefnx {Function File} {} sympref @var{cmd} @var{args}
-%% Preferences for the OctSymPy symbolic computing package.
+%% @deftypefn  Command {} sympref @var{cmd}
+%% @deftypefnx Command {} sympref @var{cmd} @var{args}
+%% @deftypefnx Function {@var{r} =} sympref ()
+%% @deftypefnx Function {@var{r} =} sympref (@var{cmd})
+%% @deftypefnx Function {@var{r} =} sympref (@var{cmd}, @var{args})
+%% Preferences for the Symbolic package.
 %%
 %% @code{sympref} can set or get various preferences and
 %% configurations.  The various choices for @var{cmd} and
 %% @var{args} are documented below.
 %%
 %%
-%% @strong{Python executable} path/command:
+%% Get the name of the @strong{Python executable}:
 %% @example
-%% @group
-%% >> sympref python '/usr/bin/python'       @c doctest: +SKIP
-%% >> sympref python 'C:\Python\python.exe'  @c doctest: +SKIP
-%% >> sympref python 'N:\myprogs\py.exe'     @c doctest: +SKIP
-%% @end group
+%% @comment doctest: +SKIP
+%% sympref python
+%%   @result{} ans = python
 %% @end example
-%% Default is an empty string; in which case OctSymPy just runs
-%% @code{python} and assumes the path is set appropriately.
+%%
+%% This value can be changed by setting the environment variable
+%% @code{PYTHON}, which can be configured in the OS, or it can be
+%% set within Octave using:
+%% @example
+%% @comment doctest: +SKIP
+%% setenv PYTHON /usr/bin/python
+%% setenv PYTHON C:\Python\python.exe
+%% sympref reset
+%% @end example
+%% (In older versions, @code{sympref python /bin/python} could be
+%% used: this is deprecated and will be removed in a future version.)
+%%
+%% If the environment variable is empty or not set, the package
+%% uses a default setting (usually @code{python}).
+%%
 %%
 %% @strong{Display} of syms:
+%%
 %% @example
 %% @group
-%% >> sympref display
-%%    @result{} ans = unicode
+%% sympref display
+%%   @result{} ans = unicode
+%%
 %% @end group
 %% @group
-%% >> syms x
-%% >> sympref display flat
-%% >> sin(x/2)
-%%    @result{} (sym) sin(x/2)
+%% syms x
+%% sympref display flat
+%% sin(x/2)
+%%   @result{} (sym) sin(x/2)
 %%
-%% >> sympref display ascii
-%% >> sin(x/2)
-%%    @result{} (sym)
-%%              /x\
-%%           sin|-|
-%%              \2/
+%% sympref display ascii
+%% sin(x/2)
+%%   @result{} (sym)
+%%          /x\
+%%       sin|-|
+%%          \2/
 %%
-%% >> sympref display unicode
-%% >> sin(x/2)
-%%    @result{} (sym)
-%%              ⎛x⎞
-%%           sin⎜─⎟
-%%              ⎝2⎠
+%% sympref display unicode
+%% sin(x/2)
+%%   @result{} (sym)
+%%          ⎛x⎞
+%%       sin⎜─⎟
+%%          ⎝2⎠
 %%
-%% >> sympref display default
+%% sympref display default
 %% @end group
 %% @end example
-%% By default OctSymPy uses the unicode pretty printer to display
+%%
+%% By default, a unicode pretty printer is used to display
 %% symbolic expressions.  If that doesn't work (e.g., if you
 %% see @code{?} characters) then try the @code{ascii} option.
 %%
+%%
 %% @strong{Communication mechanism}:
+%%
 %% @example
-%% >> sympref ipc
-%%    @result{} ans = default
+%% @group
+%% sympref ipc
+%%   @result{} ans = default
+%% @end group
 %% @end example
+%%
 %% The default will typically be the @code{popen2} mechanism which
 %% uses a pipe to communicate with Python and should be fairly fast.
 %% If that doesn't work, try @code{sympref display system} which is
@@ -101,47 +122,64 @@
 %% [for debugging, may not be supported long-term].
 %% @end itemize
 %%
+%%
 %% @strong{Reset}: reset the SymPy communication mechanism.  This can be
 %% useful after an error occurs and the connection with Python
 %% becomes confused.
-%% @example
-%% >> sympref reset     @c doctest: +SKIP
-%% @end example
 %%
-%% @strong{Snippets}: when displaying a sym object, we quote the SymPy
-%% representation (or a small part of it):
 %% @example
 %% @group
-%% >> syms x
-%% >> y = [pi x];
-%% >> sympref snippet on
-%% >> y
-%%    @result{} y = (sym 1×2 matrix)       “...([[pi, Symbol('x')]])”
-%%        [π  x]
-%% >> sympref snippet off
-%% >> y
-%%    @result{} y = (sym) [π  x]  (1×2 matrix)
-%% >> sympref snippet default
+%% sympref reset                              % doctest: +SKIP
 %% @end group
 %% @end example
 %%
+%%
+%% @strong{Snippets}: when displaying a sym object, we can optionally
+%% quote a small part of the SymPy representation:
+%%
+%% @example
+%% @group
+%% syms x;  y = [pi x];
+%% sympref snippet on
+%% y
+%%   @result{} y = (sym 1×2 matrix)       “...([[pi, Symbol('x')]])”
+%%       [π  x]
+%% sympref snippet off
+%% y
+%%   @result{} y = (sym) [π  x]  (1×2 matrix)
+%% sympref snippet default
+%% @end group
+%% @end example
+%%
+%%
 %% @strong{Default precision}: control the number of digits used by
 %% variable-precision arithmetic (see also the @ref{digits} command).
+%%
 %% @example
 %% @group
-%% >> sympref digits          % get
-%%    @result{} ans = 32
-%% >> sympref digits 64       % set
-%% >> sympref digits default
+%% sympref digits          % get
+%%   @result{} ans = 32
+%% sympref digits 64       % set
+%% sympref digits default
 %% @end group
 %% @end example
 %%
-%%
-%% Report the @strong{version} number:
+%% Be @strong{quiet} by minimizing startup and diagnostics messages:
 %% @example
 %% @group
-%% >> sympref version
-%%    @result{} 2.2.4
+%% sympref quiet
+%%   @result{} ans = 0
+%% sympref quiet on
+%% sympref quiet default
+%% @end group
+%% @end example
+%%
+%% Report the @strong{version} number:
+%%
+%% @example
+%% @group
+%% sympref version
+%%   @result{} 2.4.0
 %% @end group
 %% @end example
 %%
@@ -171,10 +209,11 @@ function varargout = sympref(cmd, arg)
       sympref ('display', 'default')
       sympref ('digits', 'default')
       sympref ('snippet', 'default')
+      sympref ('quiet', 'default')
 
     case 'version'
       assert (nargin == 1)
-      varargout{1} = '2.2.4';
+      varargout{1} = '2.4.0';
 
     case 'display'
       if (nargin == 1)
@@ -213,48 +252,80 @@ function varargout = sympref(cmd, arg)
       if (nargin == 1)
         varargout{1} = settings.snippet;
       else
-	if (strcmpi(arg, 'default'))
-	  settings.snippet = false;  % Should be false for a release
-	else
+        if (strcmpi(arg, 'default'))
+          settings.snippet = false;  % Should be false for a release
+        else
           settings.snippet = tf_from_input(arg);
-	end
+        end
+      end
+
+    case 'quiet'
+      if (nargin == 1)
+        varargout{1} = settings.quiet;
+      else
+        if (strcmpi(arg, 'default'))
+          settings.quiet = false;
+        else
+          settings.quiet = tf_from_input(arg);
+        end
       end
 
     case 'python'
-      if (nargin == 1)
-        varargout{1} = settings.whichpython;
-      elseif (isempty(arg) || strcmp(arg,'[]'))
-        settings.whichpython = '';
-        sympref('reset')
-      else
-        settings.whichpython = arg;
-        sympref('reset')
+      if (nargin ~= 1)
+        %error('old syntax ''sympref python'' removed; use ''setenv PYTHON'' instead')
+        warning('OctSymPy:deprecation', ...
+          '''sympref python foo'' deprecated; use ''setenv PYTHON foo'' instead');
+        if (isempty(arg) || strcmp(arg,'[]'))
+          settings.whichpython = '';
+          sympref('reset')
+        else
+          settings.whichpython = arg;
+          sympref('reset')
+        end
+        return
       end
+      DEFAULTPYTHON = 'python';
+      pyexec = getenv('PYTHON');
+      if (isempty(pyexec))
+        pyexec = settings.whichpython;   % FIXME: deprecated; remove
+        if (isempty(pyexec))
+          pyexec = DEFAULTPYTHON;
+        end
+      end
+      varargout{1} = pyexec;
 
     case 'ipc'
       if (nargin == 1)
         varargout{1} = settings.ipc;
       else
+        verbose = ~sympref('quiet');
         sympref('reset')
         settings.ipc = arg;
         switch arg
           case 'default'
-            disp('Choosing the default [autodetect] octsympy communication mechanism')
+            msg = 'Choosing the default [autodetect] communication mechanism';
           case 'system'
-            disp('Forcing the system() octsympy communication mechanism')
+            msg = 'Forcing the system() communication mechanism';
           case 'popen2'
-            disp('Forcing the popen2() octsympy communication mechanism')
+            msg = 'Forcing the popen2() communication mechanism';
           case 'systmpfile'
-            disp('Forcing systmpfile ipc: warning: this is for debugging')
+            msg = 'Forcing systmpfile ipc: warning: this is for debugging';
           case 'sysoneline'
-            disp('Forcing sysoneline ipc: warning: this is for debugging')
+            msg = 'Forcing sysoneline ipc: warning: this is for debugging';
           otherwise
+            msg = '';
             warning('Unsupported IPC mechanism: hope you know what you''re doing')
         end
+        if (verbose)
+          disp(msg)
+        end
       end
 
     case 'reset'
-      disp('Resetting the octsympy communication mechanism');
+      verbose = ~sympref('quiet');
+      if (verbose)
+        disp('Resetting the communication mechanism');
+      end
       r = python_ipc_driver('reset', []);
 
       if (nargout == 0)
@@ -303,31 +374,66 @@ function r = tf_from_input(s)
 end
 
 
+%!test
+%! % test quiet, side effect of making following tests a bit less noisy!
+%! sympref quiet on
+%! a = sympref('quiet');
+%! assert(a == 1)
+
+%% Test for correct line numbers in Python exceptions.  These first tests
+%% happen with the default ipc---usually popen2.  We don't explicitly test
+%% popen2 because test suite should be portable.
+%!error <line 1> python_cmd('raise ValueError');
+%!error <line 1> python_cmd('raise ValueError', sym('x'));
+%!error <line 1> python_cmd('raise ValueError', sym([1 2 3; 4 5 6]));
+%!error <line 1> python_cmd('raise ValueError', {1; 1; 1});
+%!error <line 1> python_cmd('raise ValueError', struct('a', 1, 'b', 'word'));
+%!error <line 2> python_cmd( {'x = 1' 'raise ValueError'} );
+%!error <line 3> python_cmd( {'x = 1' 'pass' '1/0'} );
+%!error <line 3> python_cmd( {'a=1' 'b=1' 'raise ValueError' 'c=1' 'd=1'} );
+
+
 %!test
 %! % system should work on all system, but just runs sysoneline on windows
 %! fprintf('\nRunning some tests that reset the IPC and produce output\n');
 %! sympref('ipc', 'system');
-%! pause(1);
 %! syms x
-%! pause(2);
+
+%!error <line 1> python_cmd('raise ValueError')
+%!error <line 1> python_cmd('raise ValueError', sym('x'))
+%!error <line 1> python_cmd('raise ValueError', struct('a', 1, 'b', 'word'))
+
 
 %!test
 %! % sysoneline should work on all systems
-%! fprintf('\n');
 %! sympref('ipc', 'sysoneline');
-%! pause(1);
 %! syms x
-%! pause(2);
+
+%!error <line 1> python_cmd('raise ValueError')
+%!error <line 1> python_cmd('raise ValueError', struct('a', 1, 'b', 'word'))
+
+
+%!test
+%! sympref('ipc', 'systmpfile');
+%! syms x
+%! delete('tmp_python_cmd.py')
+
+%!error <line 1> python_cmd('raise ValueError')
+%!error <line 1> python_cmd('raise ValueError', struct('a', 1, 'b', 'word'))
+
+%!test
+%! % (just to cleanup after the error tests)
+%! delete('tmp_python_cmd.py')
 
 %!test
 %! sympref('defaults')
 %! assert(strcmp(sympref('ipc'), 'default'))
+%! sympref('quiet', 'on')
 
 %!test
-%! fprintf('\n');
 %! syms x
 %! r = sympref('reset');
-%! pause(1);
 %! syms x
-%! pause(2);
 %! assert(r)
+%! % ok, can be noisy again
+%! sympref('quiet', 'default')
