@@ -1,4 +1,4 @@
-%% Copyright (C) 2014, 2015 Colin B. Macdonald
+%% Copyright (C) 2014-2016 Colin B. Macdonald
 %%
 %% This file is part of OctSymPy.
 %%
@@ -18,9 +18,9 @@
 
 %% -*- texinfo -*-
 %% @documentencoding UTF-8
-%% @deftypefn  {Function File} {@var{y} =} vpasolve (@var{e})
-%% @deftypefnx {Function File} {@var{y} =} vpasolve (@var{e}, @var{x})
-%% @deftypefnx {Function File} {@var{y} =} vpasolve (@var{e}, @var{x}, @var{x0})
+%% @defun  vpasolve (@var{e})
+%% @defunx vpasolve (@var{e}, @var{x})
+%% @defunx vpasolve (@var{e}, @var{x}, @var{x0})
 %% Numerical solution of a symbolic equation.
 %%
 %% Variable-precision numerical solution of the equation @var{e}
@@ -29,21 +29,22 @@
 %% Example:
 %% @example
 %% @group
-%% >> syms x
-%% >> eqn = exp(x) == x + 2;
-%% >> vpasolve(eqn, x, 0.1)
-%%    @result{} (sym) 1.1461932206205825852370610285214
+%% syms x
+%% eqn = exp(x) == x + 2;
+%% vpasolve(eqn, x, 0.1)
+%%   @result{} (sym) 1.1461932206205825852370610285214
 %% @end group
 %% @end example
 %%
 %% @seealso{vpa}
-%% @end deftypefn
-
-%% Author: Colin B. Macdonald
-%% Keywords: symbolic
+%% @end defun
 
 function r = vpasolve(e, x, x0)
 
+  if (nargin < 1 || nargin > 3)
+    print_usage ();
+  end
+
   if (nargin < 3)
     x0 = sym(0);
   end
@@ -60,7 +61,7 @@ function r = vpasolve(e, x, x0)
   % the try, except bit.
   cmd = {
     '(e, x, x0, n) = _ins'
-    'if sympy.__version__ == "0.7.5" or sympy.__version__.startswith("0.7.6"):'
+    'if Version(spver) < Version("0.7.7.dev"):'
     '    try:'
     '        sympy.mpmath.mp.dps = n'
     '        findroot = sympy.mpmath.findroot'
